import contextlib
import xbmcplugin
from resources.lib.api.api import API
from resources.lib.const import COMMAND, ROUTE, LANG, SETTINGS, STRINGS, MEDIA_TYPE, DIR_TYPE, \
    ICON, a_z_threshold_options, RENDER_TYPE, FILTER, COUNT
from resources.lib.gui import MoviesItem, SettingsItem, SearchItem, DirectoryItem, TvShowsItem, WatchHistoryItem, \
    MainMenuFolderItem, DownloadQueueItem, TraktItem
from resources.lib.gui.renderers import Renderer
from resources.lib.gui.renderers.dialog_renderer import DialogRenderer
from resources.lib.kodilogging import logger
from resources.lib.routing.router import router
from resources.lib.storage.settings import settings
from resources.lib.storage.sqlite import DB
from resources.lib.utils.kodiutils import get_string, get_icon, languages_gui
from resources.lib.api.trakt_api import trakt


class Directory:
    @staticmethod
    def main_menu():

        menu = [SearchItem(url=router.get_url(ROUTE.SEARCH)),
                MoviesItem(url=router.get_url(ROUTE.MOVIES)),
                TvShowsItem(url=router.get_url(ROUTE.TV_SHOWS)),
                WatchHistoryItem(url=router.get_url(ROUTE.WATCHED)),
                DownloadQueueItem(url=router.get_url(ROUTE.DOWNLOAD_QUEUE)),
                SettingsItem(url=router.get_url(ROUTE.COMMAND, command=COMMAND.OPEN_SETTINGS))
                ]

        if trakt.is_authenticated():
          menu.insert(3, TraktItem())

        return menu

    @staticmethod
    def movies():
        return Directory.media_menu(MEDIA_TYPE.MOVIE, [
            DirectoryItem(title=get_string(LANG.CSFD_TIPS),
                          icon=get_icon(ICON.TV),
                          url=router.get_url(ROUTE.CSFD_TIPS))
        ])

    @staticmethod
    def tv_shows():
        return Directory.media_menu(MEDIA_TYPE.TV_SHOW, [])

    @staticmethod
    def media_menu(media_type, additional_items):
        return [
                   Directory.MENU_ITEM.popular(media_type),
                   Directory.MENU_ITEM.new_releases(media_type),
                   Directory.MENU_ITEM.new_releases_dubbed(media_type,
                                                           settings.get_languages(SETTINGS.PREFERRED_LANGUAGE,
                                                                                  SETTINGS.FALLBACK_LANGUAGE)),
                   Directory.MENU_ITEM.last_added(media_type),

               ] + additional_items + Directory.by_menu(media_type)

    @staticmethod
    def by_menu(media_type):
        return [Directory.MENU_ITEM.a_z(media_type),
                Directory.MENU_ITEM.genres(media_type),
                Directory.MENU_ITEM.countries(media_type),
                Directory.MENU_ITEM.languages(media_type),
                Directory.MENU_ITEM.years(media_type),
                Directory.MENU_ITEM.studios(media_type), ]

    @staticmethod
    def count_menu(data_count, icon_builder, context_menu, url_builder, count_type):
        list_items = []
        for item in data_count:
            key = item.get('key').encode('utf-8')
            list_items.append(DirectoryItem(key=key,
                                            title=item.get('title'),
                                            context_menu=[menu_item(key, count_type) for menu_item in context_menu],
                                            icon=icon_builder(key),
                                            url=url_builder(item)))
        return list_items

    @staticmethod
    def count_menu_next_page(media_type, count_type, filter_type, render_type, filter_value=STRINGS.EMPTY, page=0):
        return DirectoryItem(title=get_string(LANG.NEXT_PAGE),
                             url=router.get_url(ROUTE.COUNT_MENU,
                                                page=page,
                                                count_type=count_type,
                                                filter_type=filter_type,
                                                filter_value=filter_value,
                                                media_type=media_type,
                                                render_type=render_type),
                             )

    class MENU_ITEM:
        @staticmethod
        def count_dir(media_type, translation_id, icon, count_type, filter_type, render_type,
                      filter_value=STRINGS.EMPTY, page=0, **kwargs):
            return DirectoryItem(title=get_string(translation_id),
                                 icon=get_icon(icon),
                                 url=router.get_url(ROUTE.COUNT_MENU,
                                                    page=page,
                                                    count_type=count_type,
                                                    filter_type=filter_type,
                                                    filter_value=filter_value,
                                                    media_type=media_type,
                                                    render_type=render_type,
                                                    **kwargs),
                                 )

        @staticmethod
        def a_z(media_type, title=LANG.A_Z, icon=ICON.A_Z, digits=0):
            return Directory.MENU_ITEM.count_dir(media_type, title, icon, COUNT.TITLES,
                                                 FILTER.STARTS_WITH_SIMPLE,
                                                 RENDER_TYPE.A_Z,
                                                 digits=digits)

        @staticmethod
        def studios(media_type):
            return Directory.MENU_ITEM.count_dir(media_type, LANG.BY_STUDIO, ICON.STUDIO, COUNT.STUDIOS,
                                                 FILTER.STUDIO,
                                                 RENDER_TYPE.DEFAULT)

        @staticmethod
        def genres(media_type):
            return Directory.MENU_ITEM.count_dir(media_type, LANG.GENRE, ICON.GENRE, COUNT.GENRES, FILTER.GENRE,
                                                 RENDER_TYPE.DEFAULT)

        @staticmethod
        def countries(media_type):
            return Directory.MENU_ITEM.count_dir(media_type, LANG.BY_COUNTRY, ICON.COUNTRY, COUNT.COUNTRIES,
                                                 FILTER.COUNTRY,
                                                 RENDER_TYPE.DEFAULT)

        @staticmethod
        def languages(media_type):
            return Directory.MENU_ITEM.count_dir(media_type, LANG.BY_LANGUAGE, ICON.LANGUAGE, COUNT.LANGUAGES,
                                                 FILTER.LANGUAGE,
                                                 RENDER_TYPE.DEFAULT)

        @staticmethod
        def years(media_type):
            return Directory.MENU_ITEM.count_dir(media_type, LANG.BY_YEAR, ICON.YEAR, COUNT.YEARS, FILTER.YEAR,
                                                 RENDER_TYPE.DEFAULT)

        @staticmethod
        def new_releases(media_type):
            return DirectoryItem(title=get_string(LANG.NEWS),
                                 icon=get_icon(ICON.NEW),
                                 url=router.get_media(media_type, RENDER_TYPE.DEFAULT,
                                                      API.FILTER.new_releases(media_type)))

        @staticmethod
        def new_releases_dubbed(media_type, languages):
            title = get_string(LANG.NEWS_DUBBED)
            title = STRINGS.NEWS_DUBBED_DIR_TITLE.format(title=title, languages='/'.join(languages_gui(languages)))
            return DirectoryItem(title=title,
                                 icon=get_icon(ICON.NEW_DUB),
                                 url=router.get_media(media_type,
                                                      RENDER_TYPE.DEFAULT,
                                                      API.FILTER.new_releases_dubbed(media_type, languages)))

        @staticmethod
        def popular(media_type):
            return DirectoryItem(title=get_string(LANG.POPULAR),
                                 icon=get_icon(ICON.POPULAR),
                                 url=router.get_media(media_type,
                                                      RENDER_TYPE.DEFAULT,
                                                      API.FILTER.popular(media_type)))

        @staticmethod
        def last_added(media_type):
            return DirectoryItem(title=get_string(LANG.LAST_ADDED),
                                 icon=get_icon(ICON.RECENT),
                                 url=router.get_media(media_type,
                                                      RENDER_TYPE.DEFAULT,
                                                      API.FILTER.last_added(media_type)))


class DirectoryRenderer(Renderer):
    def __init__(self):
        super(DirectoryRenderer, self).__init__()

    def __call__(self, handle, list_items, route=None):
        pinned_items = [row[0] for row in DB.PINNED.get(route)] if route else []
        DirectoryRenderer.render(handle, list_items, pinned_items)

    @staticmethod
    @contextlib.contextmanager
    def start_directory(handle, as_type=DIR_TYPE.NONE):
        """Simple context manager that automatically ends the directory."""
        xbmcplugin.setContent(handle, as_type)
        yield
        xbmcplugin.endOfDirectory(handle, cacheToDisc=False)

    @staticmethod
    def a_to_z_menu(media_type, count_menu, letters, letter_counts, digits):
        if letters:
            count_menu.insert(0, MainMenuFolderItem(url=router.get_url(ROUTE.CLEAR_PATH)))
            search_title = get_string(LANG.SEARCH_FOR_LETTERS).format(letters=letters)
            count_menu.insert(0, SearchItem(
                title=DirectoryRenderer.TITLE.count({'title': search_title, 'doc_count': letter_counts.get('total')}),
                url=router.get_media(media_type,
                                     RENDER_TYPE.A_Z,
                                     API.FILTER.a_z(media_type, letters),
                                     letters=letters)
            ))
            count_menu.append(MainMenuFolderItem(url=router.get_url(ROUTE.CLEAR_PATH)))
        elif not int(digits) and not letters:
            count_menu.insert(0, Directory.MENU_ITEM.a_z(media_type, LANG.NUMBERS, ICON.NUMBERS, digits=1))
        return count_menu

    @staticmethod
    def render(handle, list_items, pinned_items):
        with DirectoryRenderer.start_directory(handle):
            list_items.sort(key=DirectoryRenderer._sort_pinned(pinned_items))
            for item in list_items:
                if item.key in pinned_items:
                    item.title = STRINGS.PINNED_ITEM.format(item=item.title)
                item(handle)

    @staticmethod
    def _sort_pinned(pinned):
        def helper(item):
            key = item.key
            if key in pinned:
                return pinned.index(key)
            return len(pinned)

        return helper

    class URL:
        @staticmethod
        def default(media_type, filter_name, render_type, count_type, item, **kwargs):
            key = item.get('key')
            return router.get_media(media_type,
                                    render_type,
                                    API.FILTER.filter(media_type, filter_name, key), **kwargs)

        @staticmethod
        def az(media_type, filter_type, render_type, count_type, item, **kwargs):
            key = item.get('key')
            if item.get('doc_count') <= a_z_threshold_options[settings.as_int(SETTINGS.A_Z_THRESHOLD)]:
                return DirectoryRenderer.URL.default(media_type, filter_type, render_type, count_type, item, letters=key)
            else:
                return router.get_url(ROUTE.COUNT_MENU,
                                      count_type=count_type,
                                      filter_type=filter_type,
                                      filter_value=key,
                                      media_type=media_type,
                                      render_type=render_type,
                                      **kwargs
                                      )

    class TITLE:
        @staticmethod
        def default(item):
            title = item.get('title')
            return title

        @staticmethod
        def count(item):
            title = item.get('title')
            count = item.get('doc_count')
            return STRINGS.COUNT_TITLE.format(title, str(count))

        @staticmethod
        def pinned(pinned_keys, item):
            title = item.get('title')
            key = item.get('key')
            if key in pinned_keys:
                return STRINGS.PINNED_ITEM.format(item=title)
            return title

    # Cannot be more than 1 dir deep due to path history reset
    @staticmethod
    def search(handle, media_type):
        logger.debug('Search dialog opened')
        search_value = DialogRenderer.search()
        xbmcplugin.endOfDirectory(handle, cacheToDisc=False)
        if search_value:
            router.go(router.get_media(media_type,
                                       RENDER_TYPE.SEARCH,
                                       API.FILTER.search(media_type, search_value)))
        else:
            logger.debug('No value for search. Returning.')
            router.replace_route(handle, ROUTE.ROOT)
